from pyramid.config import Configurator
from sqlalchemy import engine_from_config

from .models import (
    DBSession,
    Base,
    )


def main(global_config, **settings):
    """ This function returns a Pyramid WSGI application.
    """
    engine = engine_from_config(settings, 'sqlalchemy.')
    DBSession.configure(bind=engine)
    Base.metadata.bind = engine
    config = Configurator(settings=settings)
    config.add_static_view('static', 'static', cache_max_age=3600)
    config.add_route('home', '/')
    config.add_route('parenturl', 'checkurl')
    config.add_route('checkpages', 'getpages')
    config.scan()
    return config.make_wsgi_app()



from sqlalchemy import (
    Column,
    Integer,
    Float,
    Text,
    Unicode,
    )

from sqlalchemy.ext.declarative import declarative_base

from sqlalchemy.orm import (
    scoped_session,
    sessionmaker,
    )

from zope.sqlalchemy import ZopeTransactionExtension

DBSession = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))
Base = declarative_base()


CSV_HEADER =['urlname', 'parentname', 'baseref', 'result', 'warningstring',
    'infostring', 'valid', 'url', 'line', 'column', 'name', 'dltime',
    'dlsize', 'checktime', 'cached', 'level', 'modified']

class CheckedLink(Base):
    __tablename__ = 'checked_links'
    urlid = Column(Integer, primary_key=True)
    urlname = Column(Unicode(512), nullable=False) #varchar(256) not null,
    parentname = Column(Unicode(512))               #varchar(256),
    baseref = Column(Unicode(80))                   #varchar(256),
    result = Column(Unicode(256))                   #varchar(256),
    warning = Column(Unicode(512))                  #varchar(512),
    info = Column(Unicode(512))                     #varchar(512),
    valid = Column(Unicode(80))                     #int,
    url = Column(Unicode(512))                      #varchar(256),
    line = Column(Integer)                          #int,
    col = Column(Integer)                           #int,
    name = Column(Unicode(256))                     #varchar(256),
    dltime = Column(Float)                        #int,
    dlsize = Column(Integer)                        #int,
    checktime = Column(Float)                     #int,
    cached = Column(Integer)                        #int,
    level = Column(Integer, nullable=False)        #int not null,
    modified = Column(Unicode(256))                 #varchar(256)

    def __init__(self, urlname, parentname, baseref,  result,
                warning, info, valid, url, line, col, name, dltime,
                dlsize, checktime, cached, level, modified=None):
        self.urlname = urlname
        self.parentname = parentname
        self.baseref = baseref
        self.valid = valid
        self.result = result
        self.warning = warning
        self.info = info
        self.url = url
        self.line = line
        self.col = col
        self.name = name
        self.checktime = checktime
        self.dltime = dltime
        self.dlsize = dlsize
        self.cached = cached
        self.level = level
        self.modified = modified




import urlparse
try:
    import simplejson as json
except ImportError:
    import json

from pyramid.response import Response
from pyramid.view import view_config
from pyramid.url import resource_url

from sqlalchemy.exc import DBAPIError

from .models import (
    DBSession,
    CheckedLink,
    )


@view_config(route_name='home', renderer='templates/mytemplate.pt')
def my_view(request):
    try:
        result = DBSession.query(CheckedLink.parentname).distinct().count()
    except DBAPIError:
        return Response(conn_err_msg, content_type='text/plain', status_int=500)
    return {'result': result, 'project': 'liches'}

conn_err_msg = """\
Pyramid is having a problem using your SQL database.  The problem
might be caused by one of the following things:

1.  You may need to run the "initialize_liches_db" script
    to initialize your database tables.  Check your virtual
    environment's "bin" directory for this script and try to run it.

2.  Your database server may not be running.  Check that the
    database server referred to by the "sqlalchemy.url" setting in
    your "development.ini" file is running.

After you fix the problem, please restart the Pyramid application to
try it again.
"""

@view_config(route_name='parenturl', renderer='templates/parenturl.pt')
def parenturl_view(request):
    parenturl = request.params.get('url')
    if not parenturl:
        return Response('Required parameter url is missing', content_type='text/plain', status_int=500)
    urlobj = urlparse.urlparse(parenturl)
    if urlobj.scheme not in ['http', 'https']:
        return Response('url must start with http:// or https://', content_type='text/plain', status_int=500)
    if not urlobj.hostname:
        return Response('Required hostname is missing', content_type='text/plain', status_int=500)
    pagename = urlparse.urlunparse([urlobj.scheme, urlobj.netloc, urlobj.path, None, None, None])
    try:
        results = DBSession.query(CheckedLink).filter_by(parentname=parenturl).distinct().all()
    except DBAPIError:
        return Response(conn_err_msg, content_type='text/plain', status_int=500)
    if results is None:
        res = {'num': 0, 'urls': [], 'name': pagename}
    else:
        urls = []
        for url in results:
            urls.append({
                'urlname': url.urlname,
                'parentname': url.parentname,
                'baseref': url.baseref,
                'valid': url.valid,
                'result': url.result,
                'warning': url.warning,
                'info': url.info,
                'url': url.url,
                'line': url.line,
                'col': url.col,
                'name': url.name,
                'checktime': url.checktime,
                'dltime': url.dltime,
                'dlsize': url.dlsize,
                'cached': url.cached,
                'level': url.level,
                'modified': url.modified,})
        res = {'num': len(results), 'urls': urls, 'name':pagename}
    if request.params.get('format') == 'json':
        response =  Response(json.dumps(res))
        response.content_type='application/json'
        return response
    else:
        return res


@view_config(route_name='checkpages', renderer='templates/pages.pt')
def checked_pages_view(request):
    parenturl = request.params.get('url')
    try:
        if parenturl:
            results = DBSession.query(CheckedLink.parentname).filter(CheckedLink.parentname.like(parenturl +'%')).distinct().all()
        else:
            results = DBSession.query(CheckedLink.parentname).distinct().all()
    except DBAPIError:
        return Response(conn_err_msg, content_type='text/plain', status_int=500)
    if results is None:
        res = {'num': 0, 'urls': [], 'name': pagename}
    else:
        urls = []
        for url in results:
            urls.append([url[0],
            resource_url(request.context, request, 'checkurl', query={'url': url[0]})])
        res = {'num': len(results), 'urls': urls, 'name':parenturl}
    if request.params.get('format') == 'json':
        response =  Response(json.dumps(res))
        response.content_type='application/json'
        return response
    else:
        return res



import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()

requires = [
    'pyramid',
    'SQLAlchemy',
    'transaction',
    'pyramid_tm',
    'pyramid_debugtoolbar',
    'zope.sqlalchemy',
    'waitress',
    'linkchecker',
    ]

setup(name='liches',
      version='0.2',
      description='Liches a server for linkchecker results',
      long_description=README + '\n\n' + CHANGES,
      classifiers=[
        "Programming Language :: Python",
        "Framework :: Pyramid",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
        "Development Status :: 3 - Alpha",
        "Topic :: Internet :: WWW/HTTP :: Site Management :: Link Checking",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        ],
      author='Christian Ledermann',
      author_email='christian.ledermann@gmail.com',
      url='https://github.com/cleder/liches',
      keywords='web wsgi bfg pylons pyramid linkchecker',
      license='GPL',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      test_suite='liches',
      install_requires=requires,
      entry_points="""\
      [paste.app_factory]
      main = liches:main
      [console_scripts]
      initialize_liches_db = liches.scripts.initializedb:main
      import_liches_csv = liches.scripts.importcsv:main
      empty_link_table = liches.scripts.emptydb:main
      """,
      )



import os
import sys
import transaction
import csv

from sqlalchemy import engine_from_config

CSV_HEADER =['urlname', 'parentname', 'baseref', 'result', 'warningstring',
    'infostring', 'valid', 'url', 'line', 'column', 'name', 'dltime',
    'dlsize', 'checktime', 'cached', 'level', 'modified']



from pyramid.paster import (
    get_appsettings,
    setup_logging,
    )

from ..models import (
    DBSession,
    CheckedLink,
    Base,
    )


def usage(argv):
    cmd = os.path.basename(argv[0])
    print('usage: %s <config_uri>\n'
          '(example: "%s development.ini")' % (cmd, cmd))
    sys.exit(1)


def main(argv=sys.argv):
    if len(argv) != 2:
        usage(argv)
    config_uri = argv[1]
    setup_logging(config_uri)
    settings = get_appsettings(config_uri)
    engine = engine_from_config(settings, 'sqlalchemy.')
    DBSession.configure(bind=engine)
    Base.metadata.create_all(engine)
    fo = open('linkchecker-out.csv', 'rU')
    reader=csv.reader(fo, delimiter=';')
    line = reader.next()
    while line[0].startswith('#'):
        line = reader.next()
    assert(line[:16] == CSV_HEADER[:16])
    with transaction.manager:
        try:
            for line in reader:
                if line[0].startswith('#'):
                    continue
                else:
                    line[10] = line[10].decode('UTF-8')
                i=0
                for l in line:
                    print i,CSV_HEADER[i],l
                    i+=1
                checked_link = CheckedLink( *line)
                DBSession.add(checked_link)
        except:
            print "Error in line: ", line
